package com.bookstore.web;

import com.bookstore.Book;
import java.io.IOException;

import javax.inject.Inject;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.bookstore.BookRepository;
import com.bookstore.JDBC;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import javax.annotation.sql.DataSourceDefinition;

@DataSourceDefinition(
        name = "java:app/jdbc/bookstore3",
        className = "org.apache.derby.jdbc.ClientDataSource",
        portNumber = 1527,
        serverName = "localhost",
        databaseName = "bookstore3",
        user = "app",
        password = "app",
        properties = {"create=true"})

@WebServlet("/book/")
public class BookListServlet extends HttpServlet {

    @Inject
    @JDBC
    private BookRepository bookRepo;
    String test;

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        if (request.getParameter("format") == null) {
            // pas de paramètre format, appel standard, on renvoir les résultats
            // via une JSP
            request.setAttribute("books", bookRepo.listBooks());
            getServletContext().getRequestDispatcher("/book-list.jsp").forward(request, response);
        } else {
            // Appel en Ajax, on envoit les résultats en JSON
            // sérialization en JSON avec la librairie Jackson
            ObjectMapper mapper = new ObjectMapper();

            String json = mapper.writeValueAsString(bookRepo.listBooks());

            // Write answer
            response.setContentType("application/json");
            response.setCharacterEncoding("UTF-8");
            response.getWriter().write(json);
        }

    }

    @Override
    protected void doPost(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {

        if (request.getParameter("title") != null) {
            // POST traditionnel
            // On récupère le contenu du formulaire
            String title = request.getParameter("title");
            String description = request.getParameter("description");
            String price = request.getParameter("price");
            String pubDate = request.getParameter("pubDate");

            // le champ "hidden" du formulaire qui contient l'id
            String id = request.getParameter("id");

            // Si l'id est null alors on est dans le cas d'un ajout de livre
            if (id == null || id.isEmpty()) {
                bookRepo.addBook(title, description, price, pubDate);
            } else {
                // Si l'id n'est pas nul on fait un update
                bookRepo.updateBook(id, title, description, price, pubDate);
            }

            // Enfin, on redirige vers la servlet qui affiche la liste des livres
            response.sendRedirect(request.getContextPath() + "/book/");
        } else {
            // Post en ajax pour ajout d'un livre
            // 1. get received JSON data from request
            BufferedReader br = new BufferedReader(new InputStreamReader(request.getInputStream()));
            String json = "";
            if (br != null) {
                json = br.readLine();
            }

            // 2. initiate jackson mapper
            ObjectMapper mapper = new ObjectMapper();

            // 3. Convert received JSON to a Book
            Book book = mapper.readValue(json, Book.class);

            // 4. Set response type to JSON
            response.setContentType("application/json");

            // 5. Add article to List<Article>
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String date = dateFormat.format(book.getPubDate());
            bookRepo.addBook(book.getTitle(), book.getDescription(),
                    (book.getPrice()).toString(), date);

            // 6. Send List<Article> as JSON to client
            mapper.writeValue(response.getOutputStream(), bookRepo.listBooks());
        }

    }
}
